@@ -35,7 +35,12 @@ def scenario(request):
     return request.param
 
 
-def test_yaml_snippet(check_job):
+def test_yaml_snippet(scenario, check_job):
     # Some tests using config with 'include_path' expect JJB root to be current directory.
     os.chdir(Path(__file__).parent / "../..")
-    check_job()
+    if scenario.name.startswith("deprecated-"):
+        with pytest.warns(UserWarning) as record:
+            check_job()
+        assert "is deprecated" in str(record[0].message)
+    else:
+        check_job()
